/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#include "audio.h"
#include "globals.h"
#include "config.h"

// Initializes the audio engine
int AudioClass::Init(bool TEnabled) {

	AudioEnabled = TEnabled;
	if(AudioEnabled) {

		// Create the device
		Device = audiere::OpenDevice();
		if(!Device) {
			irrLog->log("Error initializing sound device");
			return 0;
		}
	}

	return 1;
}

// Shuts down the audio engine
int AudioClass::Close() {

	if(AudioEnabled) {
		CloseAll();

		Device = NULL;
	}

	return 1;
}

// Deletes all the sounds
void AudioClass::CloseAll() {

	if(AudioEnabled) {

		for(SoundIterator = Sounds.begin(); SoundIterator != Sounds.end(); ++SoundIterator) {
			SoundIterator->second->stop();
			SoundIterator->second = NULL;
		}
		Sounds.clear();

	}
}

// Loads a sound file
int AudioClass::Load(const stringc &TName, const stringc &TFile, bool TStream, float TVolume, bool TRepeat) {

	if(AudioEnabled) {

		// Check for existing sounds
		SoundIterator = Sounds.find(TName);
		if(SoundIterator != Sounds.end())
			return 0;

		// Load file
		audiere::OutputStreamPtr Data(audiere::OpenSound(Device, TFile.c_str(), TStream));
		if(!Data) {
			stringc Buffer("Error loading sound file: ");
			Buffer += TFile;
			irrLog->log(Buffer.c_str());
			return 0;
		}

		// Apply global sound
		if(TStream)
			TVolume *= (float)Config::Instance().MusicVolume;
		else
			TVolume *= (float)Config::Instance().SoundVolume;

		CapVolume(&TVolume);
		Data->setVolume(TVolume);
		Data->setRepeat(TRepeat);

		// Add to map
		Sounds[TName] = Data;
	}

	return 1;
}

// Stops all sounds from playing
void AudioClass::StopAll() {

	if(AudioEnabled) {
		for(SoundIterator = Sounds.begin(); SoundIterator != Sounds.end(); ++SoundIterator) {
			SoundIterator->second->stop();
			SoundIterator->second->reset();
		}
	}
}

// Plays a sound
int AudioClass::Play(const stringc &TName) {

	if(AudioEnabled) {

		audiere::OutputStream *Data = FindSound(TName);
		if(Data == NULL)
			return 0;

		Data->play();
	}

	return 1;
}

// Stops a sound from playing
int AudioClass::Stop(const stringc &TName) {

	if(AudioEnabled) {

		audiere::OutputStream *Data = FindSound(TName);
		if(Data == NULL)
			return 0;

		Data->stop();
		Data->reset();
	}

	return 1;
}

// Searches the map for a sound and returns its sound pointer
audiere::OutputStream *AudioClass::FindSound(const stringc &TName) {

	// Search for the sound
	SoundIterator = Sounds.find(TName);
	if(SoundIterator == Sounds.end())
		return NULL;

	return SoundIterator->second.get();
}
