/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef AUDIO_H
#define AUDIO_H

// Libraries
#include <string.h>
#include <map>
#include <audiere.h>
#include <irrlicht/irrlicht.h>
#include "singleton.h"

// Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Classes
class AudioClass {

	public:

		AudioClass() { }
		~AudioClass() { }

		int Init(bool TEnabled);
		int Close();

		int Load(const stringc &TName, const stringc &TFile, bool TStream, float TVolume = 1.0f, bool TRepeat = false);
		void CloseAll();

		int Play(const stringc &TName);
		int Stop(const stringc &TName);
		void StopAll();

	private:

		audiere::OutputStream *FindSound(const stringc &TName);
		void CapVolume(float *TVolume);

		audiere::AudioDevicePtr Device;
		bool AudioEnabled;

		std::map<stringc, audiere::OutputStreamPtr> Sounds;
		std::map<stringc, audiere::OutputStreamPtr>::iterator SoundIterator;

};

// Limits a volume to 0-1
inline void AudioClass::CapVolume(float *TVolume) {
	if(*TVolume < 0.0f)
		*TVolume = 0.0f;
	else if(*TVolume > 1.0f)
		*TVolume = 1.0f;
}

// Singletons
typedef Singleton<AudioClass> Audio;

#endif
