/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#include "campaign.h"
#include "globals.h"
#include "../tinyxml/tinyxml.h"

// Loads the campaign data
int CampaignClass::Init() {
	
	Campaigns.clear();
	CurrentCampaign = 0;
	CurrentLevel = 0;

	// Open the XML file
	stringc LevelFile = stringc("campaigns/irrlamb.xml");
	TiXmlDocument Document(LevelFile.c_str());
	if(!Document.LoadFile()) {
		stringc Buffer = stringc("line ") + stringc(Document.ErrorRow()) + stringc(" column ")
						+ stringc(Document.ErrorCol()) + stringc(": ") + stringc(Document.ErrorDesc());
		irrLog->log(Buffer.c_str());
		Close();
		return 0;
	}

	// Check for level tag
	TiXmlElement *CampaignsElement = Document.FirstChildElement("campaigns");
	if(!CampaignsElement) {
		irrLog->log("Could not find campaign tag");
		return 0;
	}

	// Load campaigns
	TiXmlElement *CampaignElement = CampaignsElement->FirstChildElement("campaign");
	for(; CampaignElement != 0; CampaignElement = CampaignElement->NextSiblingElement("campaign")) {

		CampaignStruct Campaign;
		Campaign.Name = CampaignElement->Attribute("name");

		// Get levels
		TiXmlElement *LevelElement = CampaignElement->FirstChildElement("level");
		for(; LevelElement != 0; LevelElement = LevelElement->NextSiblingElement("level")) {
			LevelStruct Level;
			Level.File = LevelElement->GetText();
			Level.Icon = Level.File.subString(0, Level.File.findFirstChar(".", 1)) + stringc(".jpg");

			Level.UnlockLevel = 0;
			LevelElement->Attribute("unlock", &Level.UnlockLevel);

			Campaign.Levels.push_back(Level);
		}

		Campaigns.push_back(Campaign);
	}

	return 1;
}

// Closes the campaign system
int CampaignClass::Close() {

	Campaigns.clear();

	return 1;
}

// Increment the level counter
bool CampaignClass::IncrementLevel() {

	CurrentLevel++;
	if(CurrentLevel >= (int)Campaigns[CurrentCampaign].Levels.size())
		return false;

	return true;
}
