/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef CAMPAIGN_H
#define CAMPAIGN_H

// Libraries
#include <irrlicht/irrlicht.h>
#include <bullet/btBulletCollisionCommon.h>
#include <bullet/btBulletDynamicsCommon.h>
#include "singleton.h"

// Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Forward Declarations

// Structures

struct LevelStruct {
	stringc File, Icon;
	int UnlockLevel;
};

struct CampaignStruct {
	stringw Name;
	array<LevelStruct> Levels;
};

// Classes
class CampaignClass {

	public:

		int Init();
		int Close();

		const array<CampaignStruct> &GetCampaigns() const { return Campaigns; }
		const CampaignStruct &GetCampaign(int TIndex) const { return Campaigns[TIndex]; }

		const stringc &GetLevel() const { return Campaigns[CurrentCampaign].Levels[CurrentLevel].File; }

		bool IncrementLevel();
		void SetCurrentCampaign(int TIndex) { CurrentCampaign = TIndex; }
		void SetCurrentLevel(int TIndex) { CurrentLevel = TIndex; }

		int GetCurrentCampaign() const { return CurrentCampaign; }
		int GetCurrentLevel() const { return CurrentLevel; }

	private:

		array<CampaignStruct> Campaigns;
		int CurrentCampaign, CurrentLevel;
		
};

// Singletons
typedef Singleton<CampaignClass> Campaign;

#endif
