/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef GAME_H
#define GAME_H

// Libraries
#include <irrlicht/irrlicht.h>
#include <stack>
#include "timer.h"
#include "singleton.h"

// Namespaces
using std::stack;
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Forward Declarations
class StateClass;

class GameClass {

	// Enumerations
	enum RequestType {
		NONE,
		CHANGE,
		PUSH		
	};

	public:

		GameClass();

		int Init(int TCount, char **TArguments);
		void Update();
		void Close();
		
		int ChangeState(StateClass *TState);
		int PopState();
		StateClass *GetCurrentState();

		bool IsDone() { return Done; }
		void SetDone(bool TValue) { Done = TValue; }

		u32 GetDeltaTime() const { return DeltaTime; }
		float GetDeltaTimeSeconds() const { return DeltaTime * 0.001f; }

		bool GetQuitOnEscape() { return QuitOnEscape; }

		void Delay(u32 TTime);

	private:

		void ResetTimer();
		void ResetGraphics();

		// States
		stack<StateClass *> States;
		StateClass *NewState;
		bool PreviousWindowActive, WindowActive;

		// Flags
		RequestType ChangeType;
		bool Done, MouseWasLocked, MouseWasShown;

		// Time
		u32 TimeStamp, DeltaTime;

		// Game
		bool QuitOnEscape;

};

// Singletons
typedef Singleton<GameClass> Game;

#endif
