/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#include "graphics.h"
#include "save.h"
#include "globals.h"
#include "config.h"

// Initializes the graphics system
int GraphicsClass::Init(int TWidth, int THeight, bool TFullScreen, E_DRIVER_TYPE TDriverType, IEventReceiver *TEventReceiver) {
	
	// Create the irrlicht device
	irrDevice = createDevice(TDriverType, dimension2di(TWidth, THeight), 32, TFullScreen, true, false, NULL);
	if(irrDevice == NULL)
		return 0;

	irrDevice->setWindowCaption(L"irrlamb");
	irrDevice->setEventReceiver(TEventReceiver);

	// Save off global pointers
	irrDriver = irrDevice->getVideoDriver();
	irrScene = irrDevice->getSceneManager();
	irrGUI = irrDevice->getGUIEnvironment();
	irrFile = irrDevice->getFileSystem();
	irrTimer = irrDevice->getTimer();
	irrLog = irrDevice->getLogger();

	// Generate a list of video modes
	IVideoModeList *VideoModeList = irrDevice->getVideoModeList();
	VideoModes.clear();

	VideoModeStruct VideoMode;
	for(int i = 0; i < VideoModeList->getVideoModeCount(); i++) {
		VideoMode.Width = VideoModeList->getVideoModeResolution(i).Width;
		VideoMode.Height = VideoModeList->getVideoModeResolution(i).Height;
		VideoMode.BPP = VideoModeList->getVideoModeDepth(i);
		
		// Add the video mode
		if(VideoMode.BPP == 32 && VideoMode.Width >= 640) {
			VideoMode.String = stringw(VideoMode.Width);
			VideoMode.String += " x ";
			VideoMode.String += VideoMode.Height;

			VideoModes.push_back(VideoMode);
		}
	}

	irrLog->log("Starting irrlamb");

	DrawScene = true;
	ScreenshotRequested = false;

	return 1;
}

// Closes the graphics system
int GraphicsClass::Close() {
	
	// Close irrlicht
	irrDevice->drop();

	return 1;
}

// Erases the buffer and sets irrlicht up for the next frame
void GraphicsClass::BeginFrame() {
	irrDriver->beginScene(true, true, ClearColor);
	
	if(DrawScene)
		irrScene->drawAll();

	irrGUI->drawAll();
}

// Draws the buffer to the screen
void GraphicsClass::EndFrame() {

	irrDriver->endScene();

	// Handle screenshots
	if(ScreenshotRequested)
		CreateScreenshot();
}

// Returns the index of the current video mode
u32 GraphicsClass::GetCurrentVideoModeIndex() {

	// Find the video mode
	for(u32 i = 0; i < VideoModes.size(); i++) {
		if(Config::Instance().ScreenWidth == VideoModes[i].Width && Config::Instance().ScreenHeight == VideoModes[i].Height)
			return i;
	}

	return 0;
}

// Request screenshot
void GraphicsClass::SaveScreenshot() {

	irrGUI->getRootGUIElement()->setVisible(0);
	ScreenshotRequested = 1;
}

// Create the screenshot
void GraphicsClass::CreateScreenshot() {

	// Get time
	time_t Now;
	time(&Now);

	// Get filename
	char Filename[32];
	strftime(Filename, 32, "%Y%m%d-%H%M%S.jpg", localtime(&Now)); 

	// Create image
	IImage *Image = irrDriver->createScreenShot();
	irrDriver->writeImageToFile(Image, (Save::Instance().GetScreenshotsPath() / Filename).string().c_str()); 
	Image->drop();

	irrGUI->getRootGUIElement()->setVisible(1);
	ScreenshotRequested = 0;
}

