/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef GRAPHICS_H
#define GRAPHICS_H

// Libraries
#include "singleton.h"
#include <irrlicht/irrlicht.h>

// Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Structures
struct VideoModeStruct {

	int Width, Height, BPP;
	stringw String;
};

// Classes
class GraphicsClass {

	public:

		int Init(int TWidth, int THeight, bool TFullScreen, E_DRIVER_TYPE TDriverType, IEventReceiver *TEventReceiver);
		int Close();

		void BeginFrame();
		void EndFrame();

		void SaveScreenshot();

		void SetClearColor(const SColor &TColor) { ClearColor = TColor; }
		void SetDrawScene(bool TValue) { DrawScene = TValue; }

		const array<VideoModeStruct> &GetVideoModes() { return VideoModes; }
		u32 GetCurrentVideoModeIndex();

	private:

		void CreateScreenshot();

		// Graphics state
		SColor ClearColor;
		bool DrawScene;

		bool ScreenshotRequested;

		// Modes
		array<VideoModeStruct> VideoModes;
};

// Singletons
typedef Singleton<GraphicsClass> Graphics;

#endif
