/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#include "interface.h"
#include "globals.h"
#include "config.h"
#include "../engine/game.h"

// Initializes the graphics system
int InterfaceClass::Init() {
		
	// Load skin data
	IGUISkin *Skin = irrGUI->getSkin();

	// Select an appropriate font
	stringc File("fonts/font");
	if(Config::Instance().ScreenWidth < 800)
		File += "640.xml";
	else if(Config::Instance().ScreenWidth < 1024)
		File += "800.xml";
	else
		File += "1024.xml";

	// Set font
	MainFont = irrGUI->getFont(File.c_str());
	if(MainFont)
		Skin->setFont(MainFont);

	Skin->setColor(EGDC_BUTTON_TEXT, SColor(255, 255, 255, 255));
	Skin->setColor(EGDC_WINDOW_SYMBOL, SColor(255, 255, 255, 255));	
	//Skin->setColor(EGDC_SCROLLBAR, SColor(255, 0, 0, 255));	

	Skin->setColor(EGDC_3D_FACE, SColor(255, 0, 0, 0));
	Skin->setColor(EGDC_3D_DARK_SHADOW, SColor(255, 0, 0, 0));
	Skin->setColor(EGDC_3D_HIGH_LIGHT, SColor(255, 20, 20, 20));
	Skin->setColor(EGDC_3D_SHADOW, SColor(50, 255, 255, 255));

	Timer.Start();

	return 1;
}

// Closes the graphics system
int InterfaceClass::Close() {
	
	return 1;
}

// Clear all the GUI elements
void InterfaceClass::Clear() {

	irrGUI->clear();
	TutorialText.Text = NULL;
}

// Adds tutorial text to the screen
void InterfaceClass::SetTutorialText(const stringw &TText, float TLength) {

	// Remove last text
	if(TutorialText.Text != NULL)
		TutorialText.Text->remove();

	TutorialText.Text = irrGUI->addStaticText(TText.c_str(), GetAbsoluteRect(0.1f, 0.8f, 0.8f, 0.9f), false, true, 0, -1, 0);
	TutorialText.Text->enableOverrideColor(true);
	TutorialText.Text->setOverrideColor(SColor(255, 255, 255, 255));
	TutorialText.DeleteTime = Timer.GetSeconds() + TLength;

}

// Maintains all of the timed elements
void InterfaceClass::Update(u32 TDeltaTime) {

	Timer.Update(TDeltaTime);

	if(TutorialText.Text != NULL) {
		
		float TimeLeft = TutorialText.DeleteTime - Timer.GetSeconds();
		if(TimeLeft <= 0.0f)  {
			TutorialText.Text->remove();
			TutorialText.Text = NULL;
		}
		else if(TimeLeft < 2.0f) {
			SColor TextColor = TutorialText.Text->getOverrideColor();
			TextColor.setAlpha((u32)(255 * TimeLeft / 2.0));
			TutorialText.Text->setOverrideColor(TextColor);
		}
	}

}

// Converts milliseconds to a time string
stringw InterfaceClass::ConvertSecondsToString(u32 TTime) {

	char Buffer[32];
	sprintf(Buffer, "%.2d:%.2d:%.3d", TTime / 60000, (TTime / 1000) % 60, TTime % 1000);

	return stringw(Buffer);
}

// Converts a fraction into screen coordinates
s32 InterfaceClass::GetAbsoluteX(f32 TFraction) {

	return static_cast<s32>(irrDriver->getScreenSize().Width * TFraction);
}

// Converts a fraction into screen coordinates
s32 InterfaceClass::GetAbsoluteY(f32 TFraction) {

	return static_cast<s32>(irrDriver->getScreenSize().Height * TFraction);
}

// Converts an absolute position into a fraction
f32 InterfaceClass::GetFractionX(s32 TInteger) {

	return static_cast<f32>(TInteger) / irrDriver->getScreenSize().Width;
}

// Converts an absolute position into a fraction
f32 InterfaceClass::GetFractionY(s32 TInteger) {

	return static_cast<f32>(TInteger) / irrDriver->getScreenSize().Height;
}

// Converts a fraction into screen coordinates
rect<s32> InterfaceClass::GetAbsoluteRect(f32 TLeft, f32 TTop, f32 TRight, f32 TBottom) {

	return rect<s32>(GetAbsoluteX(TLeft), GetAbsoluteY(TTop), GetAbsoluteX(TRight), GetAbsoluteY(TBottom));
}

// Converts a fraction into screen coordinates
rect<s32> InterfaceClass::GetAbsoluteRectWH(f32 TLeft, f32 TTop, f32 TWidth, f32 THeight) {

	return rect<s32>(GetAbsoluteX(TLeft), GetAbsoluteY(TTop), GetAbsoluteX(TLeft + TWidth), GetAbsoluteY(TTop + THeight));
}

// Converts a fraction into screen coordinates and centered
rect<s32> InterfaceClass::GetAbsoluteRectCentered(f32 TLeft, f32 TTop, f32 TWidth, f32 THeight) {
	TWidth *= 0.5f;
	THeight *= 0.5f;

	return rect<s32>(GetAbsoluteX(TLeft - TWidth), GetAbsoluteY(TTop - THeight), GetAbsoluteX(TLeft + TWidth), GetAbsoluteY(TTop + THeight));
}

// Returns a rectangle of text that is centered
rect<s32> InterfaceClass::GetCenteredTextRect(const stringw &TText, f32 TTop, f32 TBottom) {

	dimension2di TextArea = MainFont->getDimension(TText.c_str());
	TextArea.Width >>= 1;
	TextArea.Width++;

	return GetAbsoluteRect(0.5f - GetFractionX(TextArea.Width), TTop, 0.5f + GetFractionX(TextArea.Width), TBottom);
}

// Returns a rectangle of text that is right-aligned within a box
rect<s32> InterfaceClass::GetRightTextRect(const stringw &TText, f32 TRight, f32 TTop, f32 TBottom) {

	dimension2di TextArea = MainFont->getDimension(TText.c_str());

	return GetAbsoluteRect(TRight - GetFractionX(TextArea.Width), TTop, TRight, TBottom);
}
