/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef INTERFACE_H
#define INTERFACE_H

// Libraries
#include "singleton.h"
#include "timer.h"
#include <irrlicht/irrlicht.h>

// Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Structures
struct TutorialTextStruct {
	IGUIStaticText *Text;
	float DeleteTime;
};

// Classes
class InterfaceClass {

	public:

		int Init();
		int Close();
		void Update(u32 TDeltaTime);
		void Clear();
	
		void SetTutorialText(const stringw &TText, float TLength);
		stringw ConvertSecondsToString(u32 TTime);

		s32 GetAbsoluteX(f32 TFraction);
		s32 GetAbsoluteY(f32 TFraction);
		f32 GetFractionX(s32 TInteger);
		f32 GetFractionY(s32 TInteger);
		rect<s32> GetAbsoluteRect(f32 TLeft, f32 TTop, f32 TRight, f32 TBottom);
		rect<s32> GetAbsoluteRectWH(f32 TLeft, f32 TTop, f32 TWidth, f32 THeight);
		rect<s32> GetAbsoluteRectCentered(f32 TLeft, f32 TTop, f32 TWidth, f32 THeight);
		rect<s32> GetCenteredTextRect(const stringw &TText, f32 TTop, f32 TBottom);
		rect<s32> GetRightTextRect(const stringw &TText, f32 TRight, f32 TTop, f32 TBottom);

	private:

		TutorialTextStruct TutorialText;
		TimerClass Timer;

		IGUIFont *MainFont;

};

// Singletons
typedef Singleton<InterfaceClass> Interface;

#endif
