/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef LEVEL_H
#define LEVEL_H

// Libraries
#include <irrlicht/irrlicht.h>
#include <bullet/btBulletCollisionCommon.h>
#include <bullet/btBulletDynamicsCommon.h>
#include "singleton.h"

// Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Forward Declarations
class TiXmlElement;
class ObjectClass;
struct DefaultObjectStruct;

// Classes
class LevelClass {

	public:

		enum CustomTemplateTypes {
			CUSTOM_GRAPPLETIP,
			CUSTOM_GRAPPLEFIXED,
			CUSTOM_GRAPPLESPRING,
			CUSTOM_COUNT,
		};

		int Init(const stringc &TFile, bool TRecord);
		int Close();

		void SetClearColor();

		void SetDefaultPosition(const vector3df &TPosition) { DefaultPosition = TPosition; }
		void SetDefaultRotation(const vector3df &TRotation) { DefaultRotation = TRotation; }
		void UpdateDefaultPosition(const vector3df &TPosition) { DefaultPosition += TPosition; }
		void UpdateDefaultRotation(const vector3df &TRotation) { DefaultRotation += TRotation; }

		ObjectClass *CreateObject(DefaultObjectStruct &TObject, bool TUseTemplateOrientation = true);
		ObjectClass *CreateObjectFromTemplateIndex(int TIndex);
		void WriteCreateEvent(int TIndex);
		int GetTemplateFromName(const stringc &TName);
		bool GetTemplateFromIndex(int TIndex, DefaultObjectStruct &TObject);

		int GetCustomObjectTemplateIndex(int TType) const { return CustomTemplates[TType]; }

	private:

		int GetObjectProperties(TiXmlElement *TObjectElement, DefaultObjectStruct &TObject);
		int LoadObjects(TiXmlElement *TElement, bool TTemplate);
		void AddBuiltInTemplates();

		// Level
		stringc LevelID;
		int LevelVersion;

		// Info
		SColor ClearColor;

		// Objects
		vector3df DefaultPosition, DefaultRotation;
		array<DefaultObjectStruct *> Templates;

		// Custom objects
		int CustomTemplates[CUSTOM_COUNT];

};

// Singletons
typedef Singleton<LevelClass> Level;

#endif
