/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef OBJECTMANAGER_H
#define OBJECTMANAGER_H

// Libraries
#include "singleton.h"
#include <irrlicht/irrlicht.h>

// Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Forward Declarations
struct ReplayEventStruct;
class ObjectClass;

// Classes
class ObjectManagerClass {

	public:

		ObjectManagerClass();

		int Init();
		int Close();

		void Update(u32 TDeltaTime);
		void UpdateReplay(u32 TDeltaTime);
		void UpdateFromMovementPacket(const ReplayEventStruct &TPacket);
		void BeginFrame();
		void EndFrame();

		ObjectClass *AddObject(ObjectClass *TObject);
		void DeleteObject(ObjectClass *TObject);
		void DeleteObjectByID(int TID);
		ObjectClass *GetObjectByName(const stringc &TName);
		ObjectClass *GetObjectByID(int TID);

		void ClearObjects();
		int GetObjectCount() const { return Objects.getSize(); }

	private:

		list<ObjectClass *> Objects;
		int NextObjectID;

};

// Singletons
typedef Singleton<ObjectManagerClass> ObjectManager;

#endif
