/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef PHYSICS_H
#define PHYSICS_H

// Libraries
#include <irrlicht/irrlicht.h>
#include <bullet/btBulletCollisionCommon.h>
#include <bullet/btBulletDynamicsCommon.h>
#include "singleton.h"

// Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Classes
class PhysicsClass {

	public:

		enum FilterTypes {
			FILTER_NONE			= 0x0,
			FILTER_DEFAULT		= 0x1,
	        FILTER_STATIC		= 0x2,
	        FILTER_KINEMATIC	= 0x4,
			FILTER_GRAPPLE		= 0x8,
	        FILTER_CAMERA		= 0x10,
			FILTER_ZONE			= 0x20,
			FILTER_BASICBODIES	= FILTER_DEFAULT | FILTER_STATIC | FILTER_KINEMATIC,
		};

		PhysicsClass();

		int Init(bool TEnabled);
		int Close();

		void Update(u32 TDeltaTime);
		bool RaycastWorld(const btVector3 &TStart, btVector3 &TEnd, btVector3 &TNormal);

		void ConvertCollisionMesh(IMesh *TMesh, btTriangleMesh *TCollisionMesh, const vector3df &TScale);
		void ConvertCollisionMeshBuffer(IMeshBuffer *TMeshBuffer, btTriangleMesh *TCollisionMesh, const vector3df &TScale);
		void GetBulletTransform(btTransform &TTransform, const vector3df &TPosition, const vector3df &TRotation);
		void QuaternionToEuler(const btQuaternion &TQuat, btVector3 &TEuler);

		void SetSimulationSpeed(float TValue) { SimulationSpeed = TValue; }
		float GetSimulationSpeed() const { return SimulationSpeed; }
		btDiscreteDynamicsWorld *GetWorld() { return World; }

		bool IsEnabled() const { return Enabled; }
		void RemoveFilter(int &TValue, int TFilter);
		void SetBodyType(int &TValue, int TFilter);

	private:

		btDefaultCollisionConfiguration *CollisionConfiguration;
		btCollisionDispatcher *Dispatcher;
		btBroadphaseInterface *BroadPhase;
		btSequentialImpulseConstraintSolver *Solver;
		btDiscreteDynamicsWorld *World;

		float SimulationSpeed;
		bool Enabled;

};

// Singletons
typedef Singleton<PhysicsClass> Physics;

#endif
