/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef SAVE_H
#define SAVE_H

// Libraries
#include "singleton.h"
#include <irrlicht/irrlicht.h>
#include <boost/filesystem/path.hpp>
#include <boost/filesystem/convenience.hpp>
#include <map>

// Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;
namespace fs = boost::filesystem;

struct SaveScoreStruct {

	SaveScoreStruct() : Time(0), Date("") { }
	SaveScoreStruct(u32 TTime, const stringc &TDate) : Time(TTime), Date(TDate) { }

	bool operator<(const SaveScoreStruct &TValue) {
		return Time < TValue.Time;
	}

	u32 Time;
	stringc Date;
};

struct SaveLevelStruct {

	SaveLevelStruct() : UnlockLevel(0), LoadCount(0), LoseCount(0), WinCount(0), PlayTime(0) { }

	int UnlockLevel;
	int LoadCount, LoseCount, WinCount;
	u32 PlayTime;
	array<SaveScoreStruct> HighScores;
};

// Classes
class SaveClass {

	public:

		int Init();
		int Close();

		const fs::path &GetSavePath() const { return SavePath; }
		const fs::path &GetReplayPath() const { return ReplayPath; }
		const fs::path &GetScreenshotsPath() const { return ScreenshotsPath; }
		const fs::path &GetConfigFile() const { return ConfigFile; }

		int GetNextReplayNumber() const { return NextReplayNumber; }
		void IncrementReplayNumber() { NextReplayNumber++; }
		void FindHighestReplayNumber();
		const stringc &GetDate();

		int LoadLevelStats();
		void SaveLevelStats();
		void UpdateLevelStats(const stringc &TLevel, const SaveLevelStruct &TStats);
		bool GetLevelStats(const stringc &TLevel, SaveLevelStruct &TStats);

		void AddScore(const stringc &TLevel, u32 TTime);
		void IncrementLevelLoadCount(const stringc &TLevel);
		void IncrementLevelLoseCount(const stringc &TLevel);
		void IncrementLevelWinCount(const stringc &TLevel);
		void IncrementLevelPlayTime(const stringc &TLevel, u32 TTime);

	private:

		// Paths
		fs::path SavePath, ReplayPath, ScreenshotsPath, ConfigFile, StatsFile;
		
		// Replays
		int NextReplayNumber;
		
		// Stats
		std::map<stringc, SaveLevelStruct> LevelStats;
		std::map<stringc, SaveLevelStruct>::iterator LevelStatsIterator;
};

// Singletons
typedef Singleton<SaveClass> Save;

#endif
