/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef SCRIPTING_H
#define SCRIPTING_H

// Libraries
#include "singleton.h"
#include <irrlicht/irrlicht.h>
#include <bullet/btBulletCollisionCommon.h>
#include <bullet/btBulletDynamicsCommon.h>
#include <lua.hpp>
#include <map>

// Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Structures
struct TimedCallbackStruct {
	float TimeStamp;
	stringc Function;
};

// Forward Declarations
class ObjectClass;

// Classes
class ScriptingClass {

	public:

		ScriptingClass();
		~ScriptingClass() { }

		int Init();
		int Close();

		void Reset();
		int LoadFile(const stringc &TFile);

		void CallFunction(const stringc &TFunctionName);
		void CallCollisionHandler(const stringc &TFunctionName, ObjectClass *TBaseObject, ObjectClass *TOtherObject);
		void CallZoneHandler(const stringc &TFunctionName, int TType, ObjectClass *TZone, ObjectClass *TObject);

		bool HandleKeyPress(EKEY_CODE TKey);
		void HandleMousePress(int TButton, int TMouseX, int TMouseY);
		void UpdateTimedCallbacks();

	private:

		static luaL_reg CameraFunctions[], ObjectFunctions[], RigidBodyFunctions[],	OrbFunctions[],
						TimerFunctions[], LevelFunctions[],	GUIFunctions[], AudioFunctions[], RandomFunctions[],
						ZoneFunctions[];

		static bool CheckArguments(lua_State *TLuaObject, int TRequired);
		static int GetArguments(lua_State *TLuaObject, int TRequired, float *TData);

		// Lua functions
		static int CameraSetYaw(lua_State *TLuaObject);
		static int CameraSetPitch(lua_State *TLuaObject);

		static int ObjectGetPointer(lua_State *TLuaObject);
		static int ObjectGetName(lua_State *TLuaObject);
		static int ObjectSetLifetime(lua_State *TLuaObject);
		static int ObjectDelete(lua_State *TLuaObject);

		static int RigidBodyStop(lua_State *TLuaObject);
		static int RigidBodySetPosition(lua_State *TLuaObject);
		static int RigidBodyGetPosition(lua_State *TLuaObject);

		static int OrbDeactivate(lua_State *TLuaObject);

		static int TimerDelayedFunction(lua_State *TLuaObject);
		static int TimerStamp(lua_State *TLuaObject);

		static int LevelLose(lua_State *TLuaObject);
		static int LevelWin(lua_State *TLuaObject);
		static int LevelGetTemplate(lua_State *TLuaObject);
		static int LevelCreateObject(lua_State *TLuaObject);
		static int LevelCreateJoint(lua_State *TLuaObject);
		static int LevelCreateSpring(lua_State *TLuaObject);
		static int LevelSetPosition(lua_State *TLuaObject);
		static int LevelSetRotation(lua_State *TLuaObject);
		static int LevelUpdatePosition(lua_State *TLuaObject);
		static int LevelUpdateRotation(lua_State *TLuaObject);

		static int GUITutorialText(lua_State *TLuaObject);

		static int RandomSeed(lua_State *TLuaObject);
		static int RandomGetFloat(lua_State *TLuaObject);
		static int RandomGetInt(lua_State *TLuaObject);
		
		void AddTimedCallback(const stringc &TFunctionName, float TTime);
		void AttachKeyToFunction(EKEY_CODE TKey, const stringc &TFunctionName);
		
		list<TimedCallbackStruct> TimedCallbacks;

		std::map<EKEY_CODE, stringc> KeyCallbacks;
		std::map<EKEY_CODE, stringc>::iterator KeyCallbacksIterator;

		lua_State *LuaObject;

};

// Singletons
typedef Singleton<ScriptingClass> Scripting;

#endif
