/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#include "basicjoint.h"
#include "../engine/globals.h"
#include "../engine/physics.h"
#include "rigidbody.h"
#include "default.h"

// Constructor
BasicJointClass::BasicJointClass(DefaultObjectStruct &TObject)
:	ObjectClass(),
	Joint(NULL) {

	// Attributes
	if(Physics::Instance().IsEnabled()) {

		if(TObject.BodyA) {

			if(TObject.BodyB) {

				// Get local frames
				btTransform TransformA, TransformB;
				TransformA = (TObject.BodyA->GetBody()->getWorldTransform().inverse() * TObject.BodyB->GetBody()->getWorldTransform());
				TransformB = btTransform::getIdentity();

				// Create joint
				btGeneric6DofConstraint *NewJoint = new btGeneric6DofConstraint(*TObject.BodyA->GetBody(), *TObject.BodyB->GetBody(), TransformA, TransformB, true);

				// Set limits
				NewJoint->setLinearLowerLimit(TObject.LinearLimit[0]);
				NewJoint->setLinearUpperLimit(TObject.LinearLimit[1]);
				NewJoint->setAngularLowerLimit(TObject.AngularLimit[0]);
				NewJoint->setAngularUpperLimit(TObject.AngularLimit[1]);
				Joint = NewJoint;
			}
			else {
				btPoint2PointConstraint *NewJoint = new btPoint2PointConstraint(*TObject.BodyA->GetBody(), btVector3(0.0f, 0.0f, 0.0f));
				Joint = NewJoint;
			}
			
			Physics::Instance().GetWorld()->addConstraint(Joint);
		}

	}

	SetProperties(TObject);
}

// Destructor
BasicJointClass::~BasicJointClass() {

	if(Joint) {
		Physics::Instance().GetWorld()->removeConstraint(Joint);

		delete Joint;
	}
}
