/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#include "grapple.h"
#include "../engine/globals.h"
#include "../engine/physics.h"
#include "../engine/random.h"
#include "player.h"
#include "default.h"

// Constructor
GrappleClass::GrappleClass(DefaultObjectStruct &TObject)
:	RigidBodyClass(TObject),
	GrappleParentObject(NULL) {

	// Graphics
	Node = irrScene->addSphereSceneNode(TObject.Radius, 32);
	if(TObject.Texture != "")
		Node->setMaterialTexture(0, irrDriver->getTexture(TObject.Texture.c_str()));
	Node->setMaterialType(EMT_TRANSPARENT_ADD_COLOR);

	// Particle system
	IParticleSystemSceneNode *ParticleSystem = irrScene->addParticleSystemSceneNode(false, Node);
	ParticleSystem->setParticleSize(dimension2d<f32>(0.25f, 0.25f));

	// Create the emitter
	IParticleEmitter *Emitter = ParticleSystem->createBoxEmitter(
		aabbox3df(-0.1f, -0.05f, -0.1f, 0.1f, 0.05f, 0.1f), 
		vector3df(0.0f, 0.0005f, 0.0f),
		60, 80,
		SColor(0, 100, 20, 165), video::SColor(0, 100, 20, 165),
		1000, 1000);
	ParticleSystem->setEmitter(Emitter);
	Emitter->drop();

	// Make the particles fade out
	IParticleAffector *ParticleAffector = ParticleSystem->createFadeOutParticleAffector();
	ParticleSystem->addAffector(ParticleAffector);
	ParticleAffector->drop();

	// Set materials
	ParticleSystem->setMaterialFlag(EMF_LIGHTING, false);
	ParticleSystem->setMaterialFlag(EMF_ZBUFFER, false);
	ParticleSystem->setMaterialTexture(0, irrDriver->getTexture("textures/fire0.png"));
	ParticleSystem->setMaterialType(EMT_TRANSPARENT_VERTEX_ALPHA);

	if(Physics::Instance().IsEnabled()) {
		
		// Orientation
		btTransform Transform;
		Physics::Instance().GetBulletTransform(Transform, TObject.Position, TObject.Rotation);
		MotionState = new btDefaultMotionState(Transform);
		
		// Create shape
		Shape = new btSphereShape(TObject.Radius);

		btVector3 LocalInertia;
		Shape->calculateLocalInertia(TObject.Mass, LocalInertia);

		RigidBody = new btRigidBody(TObject.Mass, MotionState, Shape, LocalInertia);
		RigidBody->setUserPointer((void *)this);

		Physics::Instance().GetWorld()->addRigidBody(RigidBody, PhysicsClass::FILTER_GRAPPLE, PhysicsClass::FILTER_BASICBODIES);
	}

	SetProperties(TObject);
}

// Destructor
GrappleClass::~GrappleClass() {

	Node->remove();
}

// Collision callback
void GrappleClass::HandleCollision(ObjectClass *TOtherObject, const btPersistentManifold *TContactManifold, float TNormalScale) {
	
	RigidBodyClass::HandleCollision(TOtherObject, TContactManifold, TNormalScale);

	// Check for grapple tips
	if(GrappleParentObject && TOtherObject->IsHookable()) {
		GrappleParentObject->GrappleTipHit(this, (RigidBodyClass *)(TOtherObject));
		GrappleParentObject = NULL;
	}
}

// Sets the parent object of the grapple tip
void GrappleClass::SetGrappleParent(PlayerClass *TObject) {
	GrappleParentObject = TObject;
}
