/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#include "mesh.h"
#include "../engine/globals.h"
#include "../engine/physics.h"
#include "default.h"

// Constructor
MeshClass::MeshClass(DefaultObjectStruct &TObject)
:	RigidBodyClass(TObject) {

	// Graphics
	IAnimatedMesh *AnimatedMesh = irrScene->getMesh(TObject.File.c_str());
	Node = irrScene->addOctTreeSceneNode(AnimatedMesh);
	Node->setScale(TObject.Scale);
	if(TObject.Texture != "")
		Node->setMaterialTexture(0, irrDriver->getTexture(TObject.Texture.c_str()));

	if(Physics::Instance().IsEnabled()) {
		
		// Get the collision mesh
		CollisionMesh = new btTriangleMesh();
		Physics::Instance().ConvertCollisionMesh(AnimatedMesh->getMesh(0), CollisionMesh, TObject.Scale);
		Shape = new btBvhTriangleMeshShape(CollisionMesh, true);

		// Orientation
		btTransform Transform;
		Physics::Instance().GetBulletTransform(Transform, TObject.Position, TObject.Rotation);
		MotionState = new btDefaultMotionState(Transform);
		
		// Body
		RigidBody = new btRigidBody(TObject.Mass, MotionState, Shape);
		RigidBody->setUserPointer((void *)this);
		
		Physics::Instance().GetWorld()->addRigidBody(RigidBody, TObject.CollisionGroup, TObject.CollisionMask);
	}

	SetProperties(TObject);	
}

// Destructor
MeshClass::~MeshClass() {
	
	Node->remove();

	// Delete physics
	if(Physics::Instance().IsEnabled())
		delete CollisionMesh;
}
