/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef OBJECT_H
#define OBJECT_H

// Libraries
#include <irrlicht/irrlicht.h>
#include <bullet/btBulletCollisionCommon.h>
#include <bullet/btBulletDynamicsCommon.h>
#include "../engine/timer.h"

// Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Forward Declarations
struct DefaultObjectStruct;
struct ReplayMovementStruct;

// Classes
class ObjectClass {

	public:

		enum ObjectType {
			NONE,
			SPHERE,
			BOX,
			TERRAIN,
			MESH,
			SCENE,
			WATER,
			PLAYER,
			ORB,
			ZONE,
			BASICJOINT,
			SPRINGJOINT,
			GRAPPLE,
		};

		ObjectClass();
		virtual ~ObjectClass();

		virtual void Update(u32 TDeltaTime);
		virtual void UpdateRender() { }
		virtual void UpdateReplay(u32 TDeltaTime);
		virtual void HandleCollision(ObjectClass *TOtherObject, const btPersistentManifold *TContactManifold, float TNormalScale) { }
		virtual void BeginFrame() { }
		virtual void EndFrame() { }
		virtual bool GetReplayPacket(ReplayMovementStruct *TPacket) { return false; }
		virtual bool HasMovementChanged() const { return false; }
		virtual bool IsHookable() const { return false; }

		void SetID(int TValue) { ID = TValue; }
		void SetDeleted(bool TValue) { Deleted = TValue; }
		void SetLifetime(float TValue) { Lifetime = Timer.GetSeconds() + TValue; }

		stringc GetName() const { return Name; }
		bool GetDeleted() const { return Deleted; }
		float GetLifetime() const { return Lifetime; }
		ObjectType GetType() const { return Type; }
		int GetID() const { return ID; }

	protected:

		virtual void SetProperties(DefaultObjectStruct &TObject);

		// Attributes
		stringc Name;
		ObjectType Type;
		int ID;

		// State
		bool Deleted;

		// Life
		TimerClass Timer;
		float Lifetime;
};

#endif
