/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef PLAYER_H
#define PLAYER_H

// Libraries
#include "rigidbody.h"
#include "grapple.h"
#include "../engine/camera.h"

// Classes
class PlayerClass : public RigidBodyClass {

	public:

		enum GrappleStates {
			GRAPPLE_READY,
			GRAPPLE_SHOOT,
			GRAPPLE_FLYING,
			GRAPPLE_HOOKED,
			GRAPPLE_DELETEJOINT,
			GRAPPLE_DELETECHAIN,
		};

		enum GrappleRequestStates {
			GRAPPLEREQUEST_NONE,
			GRAPPLEREQUEST_SHOOT,
			GRAPPLEREQUEST_RELEASE,
		};

		PlayerClass(DefaultObjectStruct &TObject);
		~PlayerClass();

		void Update(u32 TDeltaTime);
		bool ProcessKeyPress(int TKey);
		bool HandleMousePress(int TButton, int TMouseX, int TMouseY);
		bool HandleMouseLift(int TButton, int TMouseX, int TMouseY);
		void HandleInput();

		void SetCamera(CameraClass *TCamera) { Camera = TCamera; }
		void GrappleTipHit(RigidBodyClass *TTip, RigidBodyClass *TObject);

	private:

		void UpdateGrapple();
		bool CheckGrappleDistance();

		CameraClass *Camera;

		// Jumping
		int JumpTimer;
		float TorqueFactor, Radius;

		// Grapple
		int GrappleState, GrappleRequestState;
		list<ObjectClass *> GrappleJoints;
		GrappleClass *GrappleBody;
		bool GrappleEnabled;

		// Player stats
		float StatGrappleLength;
};

#endif
