/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef _RIGIDBODY_H
#define _RIGIDBODY_H

// Libraries
#include "object.h"
#include <irrlicht/irrlicht.h>
#include <bullet/btBulletCollisionCommon.h>
#include <bullet/btBulletDynamicsCommon.h>

// Namespaces
using namespace irr;
using namespace core;
using namespace scene;
using namespace video;
using namespace io;
using namespace gui;

// Classes
class RigidBodyClass : public ObjectClass {

	public:

		RigidBodyClass(DefaultObjectStruct &TObject);
		virtual ~RigidBodyClass();

		virtual void Update(u32 TDeltaTime);
		virtual void UpdateRender();
		virtual void UpdateReplay(u32 TDeltaTime);
		virtual void BeginFrame();
		virtual bool GetReplayPacket(ReplayMovementStruct *TPacket);
		virtual bool HasMovementChanged() const { return MovementChanged; }
		virtual bool IsHookable() const { return Hookable; }

		void Stop();

		void SetPosition(const btVector3 &TPosition);
		btVector3 GetPosition();

		void SetLinearVelocity(const btVector3 &TPosition);
		btVector3 GetLinearVelocity();

		btRigidBody *GetBody() { return RigidBody; }
		ISceneNode *GetNode() { return Node; }

		virtual void HandleCollision(ObjectClass *TOtherObject, const btPersistentManifold *TContactManifold, float TNormalScale);
		bool IsTouchingGround() const { return TouchingGround; }

	protected:

		virtual void SetProperties(DefaultObjectStruct &TObject);

		// Physics and graphics
		ISceneNode *Node;
		btRigidBody *RigidBody;
		btCollisionShape *Shape;
		btDefaultMotionState *MotionState;

		// Replays
		vector3df PreviousPosition, PreviousRotation;
		bool MovementChanged;

		// Collision
		stringc CollisionCallback;
		bool TouchingGround, TouchingWall, Hookable;

};

#endif
