/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#include "scene.h"
#include "../engine/globals.h"
#include "../engine/physics.h"
#include "default.h"

// Constructor
SceneClass::SceneClass(DefaultObjectStruct &TObject)
:	RigidBodyClass(TObject) {

	// Graphics
	irrScene->loadScene(TObject.File.c_str());
	Node = irrScene->getSceneNodeFromName("world0");
	IMeshSceneNode *MeshSceneNode = (IMeshSceneNode *)Node;

	if(Physics::Instance().IsEnabled()) {

		// Get the collision mesh
		CollisionMesh = new btTriangleMesh();
		Physics::Instance().ConvertCollisionMesh(MeshSceneNode->getMesh(), CollisionMesh, vector3df(1.0f, 1.0f, 1.0f));
		Shape = new btBvhTriangleMeshShape(CollisionMesh, true);

		// Orientation
		btTransform Transform;
		Physics::Instance().GetBulletTransform(Transform, TObject.Position, TObject.Rotation);
		MotionState = new btDefaultMotionState(Transform);
		
		// Body
		RigidBody = new btRigidBody(TObject.Mass, MotionState, Shape);
		RigidBody->setUserPointer((void *)this);
		
		Physics::Instance().GetWorld()->addRigidBody(RigidBody, TObject.CollisionGroup, TObject.CollisionMask);
	}
	SetProperties(TObject);	
}

// Destructor
SceneClass::~SceneClass() {
	
	Node->remove();

	// Delete physics
	if(Physics::Instance().IsEnabled())
		delete CollisionMesh;

}
