/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#include "sphere.h"
#include "../engine/globals.h"
#include "../engine/physics.h"
#include "default.h"

// Constructor
SphereClass::SphereClass(DefaultObjectStruct &TObject)
:	RigidBodyClass(TObject) {

	// Graphics
	Node = irrScene->addSphereSceneNode(TObject.Radius, 32);
	if(TObject.Texture != "")
		Node->setMaterialTexture(0, irrDriver->getTexture(TObject.Texture.c_str()));

	if(Physics::Instance().IsEnabled()) {
		
		// Orientation
		btTransform Transform;
		Physics::Instance().GetBulletTransform(Transform, TObject.Position, TObject.Rotation);
		MotionState = new btDefaultMotionState(Transform);
		
		// Create shape
		Shape = new btSphereShape(TObject.Radius);

		btVector3 LocalInertia;
		Shape->calculateLocalInertia(TObject.Mass, LocalInertia);

		RigidBody = new btRigidBody(TObject.Mass, MotionState, Shape, LocalInertia);
		RigidBody->setUserPointer((void *)this);

		Physics::Instance().GetWorld()->addRigidBody(RigidBody, TObject.CollisionGroup, TObject.CollisionMask);
	}

	SetProperties(TObject);
}

// Destructor
SphereClass::~SphereClass() {

	Node->remove();
}
