/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#include "terrain.h"
#include "../engine/globals.h"
#include "../engine/physics.h"
#include "default.h"

// Constructor
TerrainClass::TerrainClass(DefaultObjectStruct &TObject)
:	RigidBodyClass(TObject) {

	// Graphics
	ITerrainSceneNode *Terrain = irrScene->addTerrainSceneNode(TObject.File.c_str(), 0, -1, TObject.Position, vector3df(0.0f, 0.0f, 0.0f), TObject.Scale, SColor(255, 255, 255, 255), 5, ETPS_17, TObject.SmoothFactor);
	Terrain->setMaterialType(EMT_DETAIL_MAP);
	Terrain->scaleTexture(1.0f, 40.0f);
	if(TObject.Texture != "")
		Terrain->setMaterialTexture(0, irrDriver->getTexture(TObject.Texture.c_str()));
	if(TObject.DetailTexture != "")
		Terrain->setMaterialTexture(1, irrDriver->getTexture(TObject.DetailTexture.c_str()));
	Terrain->setMaterialFlag(EMF_FOG_ENABLE, true);
	Node = Terrain;

	CDynamicMeshBuffer MeshBuffer(video::EVT_2TCOORDS, video::EIT_32BIT);
	Terrain->getMeshBufferForLOD(MeshBuffer, 0);

	if(Physics::Instance().IsEnabled()) {

		// Get the collision mesh
		CollisionMesh = new btTriangleMesh();
		Physics::Instance().ConvertCollisionMeshBuffer((IMeshBuffer *)(&MeshBuffer), CollisionMesh, TObject.Scale);
		Shape = new btBvhTriangleMeshShape(CollisionMesh, true);

		// Orientation
		btTransform Transform;
		Physics::Instance().GetBulletTransform(Transform, TObject.Position, TObject.Rotation);
		MotionState = new btDefaultMotionState(Transform);
		
		// Body
		RigidBody = new btRigidBody(TObject.Mass, MotionState, Shape);
		RigidBody->setUserPointer((void *)this);
		
		Physics::Instance().GetWorld()->addRigidBody(RigidBody, TObject.CollisionGroup, TObject.CollisionMask);
	}

	SetProperties(TObject);
}

// Destructor
TerrainClass::~TerrainClass() {
	
	Node->remove();

	// Delete physics
	if(Physics::Instance().IsEnabled())
		delete CollisionMesh;
}
