/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#include "water.h"
#include "../engine/globals.h"
#include "../engine/physics.h"
#include "default.h"

// Constructor
WaterClass::WaterClass(DefaultObjectStruct &TObject)
:	RigidBodyClass(TObject) {

	// Graphics
	IAnimatedMesh *Mesh = irrScene->addHillPlaneMesh(TObject.Name.c_str(),
		dimension2df(TObject.Scale.X, TObject.Scale.Z),
		dimension2d<u32>(128,128), 0, 0,
		dimension2df(0, 0),
		dimension2df(15, 15));

	Node = irrScene->addWaterSurfaceSceneNode(Mesh->getMesh(0), 0.2f, 300.0f, 3.0f);
	if(TObject.DetailTexture != "")
		Node->setMaterialTexture(0, irrDriver->getTexture(TObject.DetailTexture.c_str()));
	if(TObject.Texture != "")
		Node->setMaterialTexture(1, irrDriver->getTexture(TObject.Texture.c_str()));
	Node->setMaterialType(EMT_REFLECTION_2_LAYER);

	irrScene->getMeshCache()->removeMesh(Mesh);

	// Build quad
	btVector3 TopLeft(-TObject.Scale.X, 0, -TObject.Scale.Z), TopRight(TObject.Scale.X, 0, -TObject.Scale.Z), BottomLeft(-TObject.Scale.X, 0, TObject.Scale.Z), BottomRight(TObject.Scale.X, 0, TObject.Scale.Z);
	TopLeft *= 64;
	TopRight *= 64;
	BottomLeft *= 64;
	BottomRight *= 64;

	// Get the collision mesh
	if(Physics::Instance().IsEnabled()) {
		CollisionMesh = new btTriangleMesh();
		CollisionMesh->addTriangle(TopLeft, BottomRight, BottomLeft);
		CollisionMesh->addTriangle(TopLeft, BottomRight, TopRight);
		Shape = new btBvhTriangleMeshShape(CollisionMesh, true);

		// Orientation
		btTransform Transform;
		Physics::Instance().GetBulletTransform(Transform, TObject.Position, TObject.Rotation);
		MotionState = new btDefaultMotionState(Transform);
		
		// Body
		RigidBody = new btRigidBody(TObject.Mass, MotionState, Shape);
		RigidBody->setUserPointer((void *)this);
		
		Physics::Instance().GetWorld()->addRigidBody(RigidBody, TObject.CollisionGroup, TObject.CollisionMask);
	}

	SetProperties(TObject);
}

// Destructor
WaterClass::~WaterClass() {
	
	Node->remove();

	// Delete physics
	if(Physics::Instance().IsEnabled())
		delete CollisionMesh;
}
