/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#include "zone.h"
#include "../engine/globals.h"
#include "../engine/physics.h"
#include "../engine/scripting.h"
#include "default.h"

// Constructor
ZoneClass::ZoneClass(DefaultObjectStruct &TObject)
:	ObjectClass() {

	// Attributes
	Active = TObject.Active;
	CollisionCallback = TObject.CollisionCallback;

	if(Physics::Instance().IsEnabled()) {

		// Orientation
		btTransform Transform;
		Physics::Instance().GetBulletTransform(Transform, TObject.Position, TObject.Rotation);
		MotionState = new btDefaultMotionState(Transform);
		
		// Create shape
		btVector3 HalfExtents(TObject.Scale.X * 0.5f, TObject.Scale.Y * 0.5f, TObject.Scale.Z * 0.5f);
		Shape = new btBoxShape(HalfExtents);

		RigidBody = new btRigidBody(0.0f, MotionState, Shape);
		RigidBody->setCollisionFlags(btCollisionObject::CF_NO_CONTACT_RESPONSE);
		RigidBody->setUserPointer((void *)this);

		Physics::Instance().GetWorld()->addRigidBody(RigidBody, PhysicsClass::FILTER_ZONE, TObject.CollisionMask);
	}

	SetProperties(TObject);
}

// Destructor
ZoneClass::~ZoneClass() {

}

// Collision callback
void ZoneClass::HandleCollision(ObjectClass *TOtherObject, const btPersistentManifold *TContactManifold, float TNormalScale) {

	if(Active) {

		// Search for existing objects in the touch list
		for(list<ObjectTouchState>::Iterator Iterator = TouchState.begin(); Iterator != TouchState.end(); ++Iterator) {
			if((*Iterator).Object == TOtherObject) {

				// Update touch count
				(*Iterator).TouchCount = 2;
				return;
			}
		}

		// A new object has collided with the zone
		TouchState.push_back(ObjectTouchState(TOtherObject, 2));

		// Call Lua function
		if(CollisionCallback.size())
			Scripting::Instance().CallZoneHandler(CollisionCallback, 0, this, TOtherObject);
	}
}

// Removes old objects from the touch list
void ZoneClass::EndFrame() {
	
	if(Active) {
	
		// Search for old objects
		for(list<ObjectTouchState>::Iterator Iterator = TouchState.begin(); Iterator != TouchState.end(); ) {
			(*Iterator).TouchCount--;
			if((*Iterator).TouchCount <= 0) {
				
				// Call Lua function
				if(CollisionCallback.size())
					Scripting::Instance().CallZoneHandler(CollisionCallback, 1, this, (*Iterator).Object);

				Iterator = TouchState.erase(Iterator);
			}
			else
				++Iterator;
		}
	}
}

// Sets the active state of the zone
void ZoneClass::SetActive(bool TValue) {
	Active = TValue;

	TouchState.clear();
}

