/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef PLAY_H
#define PLAY_H

// Libraries
#include "engine/state.h"
#include "engine/timer.h"

// Forward Declarations
class ObjectClass;
class PlayerClass;
class CameraClass;

// Classes
class PlayState : public StateClass {

	public:

		enum StateType {
			STATE_PLAY,
			STATE_PAUSED,
			STATE_SAVEREPLAY,
			STATE_LOSE,
			STATE_WIN
		};

		enum GUIElements {
			PAUSE_RESUME,
			PAUSE_SAVEREPLAY,
			PAUSE_MAINMENU,
			SAVEREPLAY_NAME,
			SAVEREPLAY_SAVE,
			SAVEREPLAY_CANCEL,
			LOSE_RESTARTLEVEL,
			LOSE_SAVEREPLAY,
			LOSE_MAINMENU,
			WIN_RESTARTLEVEL,
			WIN_NEXTLEVEL,
			WIN_SAVEREPLAY,
			WIN_MAINMENU,
		};

		PlayState();

		int Init();
		int Close();

		bool HandleKeyPress(EKEY_CODE TKey);
		bool HandleKeyLift(EKEY_CODE TKey);
		void HandleMouseMotion(float TUpdateX, float TUpdateY);
		bool HandleMousePress(int TButton, int TMouseX, int TMouseY);
		void HandleMouseLift(int TButton, int TMouseX, int TMouseY);
		void HandleMouseWheel(float TDirection);
		void HandleGUI(EGUI_EVENT_TYPE TEventType, IGUIElement *TElement);

		void Update(u32 TDeltaTime);
		void Draw();
		
		void InitLose();
		void InitWin();
		void SetNextLevel(const stringc &TLevel) { NextLevel = TLevel; }
		const stringc &GetCurrentLevel() { return CurrentLevel; }

		CameraClass *GetCamera() { return Camera; }
		TimerClass &GetTimer() { return Timer; }

		static PlayState *Instance() {
			static PlayState ClassInstance;
			return &ClassInstance;
		}

	private:

		void InitPlay();
		void InitPause();
		void InitSaveReplay();
		void SaveReplay();

		// States
		StateType State, TargetState;
		stringc CurrentLevel, NextLevel;
		TimerClass Timer;

		// Objects
		PlayerClass *Player;
		CameraClass *Camera;

		// GUI
		IGUIStaticText *HUDText;
};

#endif
