/*************************************************************************************
*	irrlamb - http://code.google.com/p/irrlamb/
*	Copyright (C) 2007  Alan Witkowski
*
*	This program is free software: you can redistribute it and/or modify
*	it under the terms of the GNU General Public License as published by
*	the Free Software Foundation, either version 3 of the License, or
*	(at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program.  If not, see <http://www.gnu.org/licenses/>.
**************************************************************************************/
#ifndef VIEWREPLAY_H
#define VIEWREPLAY_H

// Libraries
#include "engine/state.h"
#include "engine/replay.h"
#include "engine/timer.h"

// Forward Declarations
class ObjectClass;
class PlayerClass;
class CameraClass;

// Classes
class ViewReplayState : public StateClass {

	public:

		int Init();
		int Close();

		bool HandleKeyPress(EKEY_CODE TKey);
		bool HandleKeyLift(EKEY_CODE TKey) { return false; }
		void HandleMouseMotion(float TUpdateX, float TUpdateY);
		bool HandleMousePress(int TButton, int TMouseX, int TMouseY);
		void HandleMouseLift(int TButton, int TMouseX, int TMouseY) { }
		void HandleMouseWheel(float TDirection);
		void HandleGUI(EGUI_EVENT_TYPE TEventType, IGUIElement *TElement) { }

		void Update(u32 TDeltaTime);
		void Draw();
		
		void SetCurrentReplay(const stringc &TFile) { CurrentReplay = TFile; }

		static ViewReplayState *Instance() {
			static ViewReplayState ClassInstance;
			return &ClassInstance;
		}

	private:

		// States
		stringc CurrentReplay;
		TimerClass Timer;
		IGUIStaticText *HUDText;

		// Objects
		PlayerClass *Player;
		CameraClass *Camera;
	
		// Replay information
		ReplayHeaderStruct Header;
		ReplayEventStruct NextEvent;
		float ReplaySpeed, TimeAccumulator;
};

#endif
